﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics.Contracts;

namespace tpts.Logging
{
    /// <summary>
    /// Class to filter log entries that match the specified category.
    /// </summary>
    /// <typeparam name="T">The type of item logged.</typeparam>
    /// <remarks>
    /// Class is immutable.
    /// 
    /// Contract: invar : Category is not null.
    /// </remarks>
    public class CategoryFilter<T> : ILogEntryFilter<T>
    {

        #region Declarations

        private readonly string _Category;

        #endregion

        #region Constructors

        /// <summary>
        /// Constructor for category filter, specifying the category to filter on.
        /// </summary>
        /// <param name="category"></param>
        /// <remarks>
        /// Contract: pre  : Category must not be null.
        /// </remarks>
        public CategoryFilter(string category)
        {
            Contract.Requires(category != null);

            _Category = category;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Get the category for the filter.
        /// </summary>
        /// <remarks>
        /// Contract: post : get Category is not null.
        /// </remarks>
        public virtual string Category
        {
            [Pure()]
            get
            {
                Contract.Ensures(Contract.Result<string>() != null);

                return _Category;
            }
        }

        #endregion

        #region Public Methods
        #endregion

        #region Protected Methods
        #endregion

        #region Private Methods
        #endregion

        #region ILogEntryFilter

        /// <summary>
        /// Return true if the log entry category equals the filter category based on the current culture settings.
        /// </summary>
        /// <param name="logEntry"></param>
        /// <returns>A boolean indicating if the log entry log entry category equals the filter category.</returns>
        /// <remarks>
        /// Note: If the filter category is blank then all log entries are classified as matching.
        /// 
        /// Contract: pre  : logEntry must not be null.
        /// </remarks>
        public virtual bool Accept(LogEntry<T> logEntry)
        {
            return (Category.Length == 0 || Category.Equals(logEntry.Category, StringComparison.CurrentCulture));
        }

        #endregion

        #region Contracts

        [ContractInvariantMethod]
        private void ObjectInvariant()
        {
            Contract.Invariant(Category != null);
            Contract.Invariant(_Category != null);
        }

        #endregion
    }
}
