﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics.Contracts;
using System.Diagnostics.CodeAnalysis;

namespace tpts.Logging
{
    /// <summary>
    /// Log items to any number of loggers contained within.
    /// </summary>
    /// <typeparam name="T">The type of item to be logged.</typeparam>
    public class CompositeLogger<T> : Logger<T>
    {

        #region Declarations

        private ICollection<ILogger<T>> _Loggers;

        private object _LoggersLockObject = new object(); // object used to synchronise access to _Loggers collection.

        #endregion

        #region Constructors

        /// <summary>
        /// Constructor for Composite Logger.
        /// </summary>
        [SuppressMessage("Microsoft.Contracts", "Invariant")]
        public CompositeLogger()
        {
            Contract.Ensures(_Loggers != null);

            _Loggers = CreateLoggersCollection();
            Loggers = _Loggers;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Get/Set the Loggers collection.
        /// </summary>
        /// <remarks>
        /// Contract: pre  : set value must not be null
        ///           post : get Loggers will not be null.
        /// </remarks>
        protected virtual ICollection<ILogger<T>> Loggers
        {
            get
            {
                Contract.Ensures(Contract.Result<ICollection<ILogger<T>>>() != null);

                return _Loggers;
            }
            set
            {
                Contract.Requires(value != null);

                _Loggers = value;
            }
        }

        /// <summary>
        /// Get/Set the lock object used to synch access to Loggers collection.
        /// </summary>
        /// <remarks>
        /// Contract: pre  : set value must not be null
        ///           post : get LoggersLockObject will not be null.
        /// </remarks>
        protected virtual object LoggersLockObject
        {
            get
            {
                Contract.Ensures(Contract.Result<object>() != null);

                return _LoggersLockObject;
            }
            set
            {
                Contract.Requires(value != null);

                _LoggersLockObject = value;
            }
        }

        /// <summary>
        /// Return the number of loggers contained in the composite logger.
        /// </summary>
        /// <remarks>
        /// Contract: post : count is greater or equal to zero.
        /// </remarks>
        public int Count
        {
            get
            {
                Contract.Ensures(Contract.Result<int>() >= 0);

                lock (LoggersLockObject)
                {
                    return Loggers.Count;
                }
            }
        }

        /// <summary>
        /// Get the formatter to use for this logger. Set throws unsupported exception.
        /// </summary>
        /// <exception cref="NotSupportedException"/>
        public override ILogEntryFormatter<T> LogEntryFormatter
        {
            protected get
            {
                return null;
            }
            set
            {
                throw new NotSupportedException("Setting the Log Entry Formatter is not supported for a Composite Logger");
            }
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// Clear the collection of loggers.
        /// </summary>
        /// <remarks>
        /// Contract: post :  Loggers count will be zero.
        /// </remarks>
        public virtual void Clear()
        {
            ClearLoggers();
        }

        /// <summary>
        /// Return true if the collection of loggers contains the specified logger.
        /// </summary>
        /// <param name="logger"></param>
        /// <returns></returns>
        /// <remarks>
        /// Contract: pre  : logger must not be null.
        /// </remarks>
        [Pure()]
        public virtual bool Contains(ILogger<T> logger)
        {
            Contract.Requires(logger != null);

            lock (LoggersLockObject)
            {
                return Loggers.Contains(logger);
            }
        }

        /// <summary>
        /// Add the logger to the collection of loggers.
        /// </summary>
        /// <param name="logger"></param>
        /// <returns></returns>
        /// <remarks>
        /// Contract: pre  : logger must not be null.
        ///           post : logger will be in collection.
        ///                  returns true if logger was added to collection.
        ///                  returns false if logger already existed in collection.
        /// </remarks>
        public virtual bool Add(ILogger<T> logger)
        {
            Contract.Requires(logger != null);

            bool result = false;

            if (!ContainsLogger(logger))
            {
                AddLogger(logger);
                result = true;
            }

            return result;
        }

        /// <summary>
        /// Remove the logger from the collection.
        /// </summary>
        /// <param name="logger"></param>
        /// <remarks>
        /// Contract: pre  : logger must not be null.
        ///                : logger must exist in collection.
        ///           post : logger will not be in collection.
        /// </remarks>
        public virtual void Remove(ILogger<T> logger)
        {
            Contract.Requires(logger != null);
            Contract.Requires(Contains(logger));
            Contract.Ensures(!Contains(logger));

            RemoveLogger(logger);
        }

        #endregion

        #region Protected Methods

        /// <summary>
        /// Return an empty collection to contain logger objects.
        /// </summary>
        /// <returns></returns>
        /// <remarks>
        /// Contract: post : returned collection is not null
        ///                  returned collection is empty, i.e. count = 0.
        /// </remarks>
        protected virtual ICollection<ILogger<T>> CreateLoggersCollection()
        {
            Contract.Ensures(Contract.Result<ICollection<ILogger<T>>>() != null);
            Contract.Ensures(Contract.Result<ICollection<ILogger<T>>>().Count == 0);

            return new List<ILogger<T>>();
        }

        /// <summary>
        /// Returns true if the collection of loggers contains the specified logger.
        /// </summary>
        /// <param name="logger"></param>
        /// <returns></returns>
        /// <remarks>
        /// Contract: pre  : logger must not be null.
        /// </remarks>
        [Pure()]
        protected virtual bool ContainsLogger(ILogger<T> logger)
        {
            Contract.Requires(logger != null);

            bool result = false;

            lock (LoggersLockObject)
            {
                result = Loggers.Contains(logger);
            }

            return result;
        }

        /// <summary>
        /// Add the specified logger to the collection.
        /// </summary>
        /// <param name="logger"></param>
        /// <remarks>
        /// Contract: pre  : logger must not be null.
        /// </remarks>
        protected virtual void AddLogger(ILogger<T> logger)
        {
            Contract.Requires(logger != null);

            lock (LoggersLockObject)
            {
                Loggers.Add(logger);
            }
        }

        /// <summary>
        /// Remove the specified logger to the collection.
        /// </summary>
        /// <param name="logger"></param>
        /// <remarks>
        /// Contract: pre  : logger must not be null.
        ///           post : logger will not be in collection.
        /// </remarks>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Contracts", "Ensures")]
        protected virtual void RemoveLogger(ILogger<T> logger)
        {
            Contract.Requires(logger != null);
            Contract.Ensures(!Contains(logger));

            lock (LoggersLockObject)
            {
                Loggers.Remove(logger);
            }
        }

        /// <summary>
        /// Clear the collection of loggers.
        /// </summary>
        /// <remarks>
        /// Contract: post :  _Loggers count will be zero.
        /// </remarks>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Contracts", "Ensures")]
        protected virtual void ClearLoggers()
        {
            Contract.Ensures(Loggers.Count == 0);

            lock (LoggersLockObject)
            {
                Loggers.Clear();
            }
        }

        #endregion

        #region Logger

        /// <summary>
        /// Log the entry to all the contained loggers.
        /// </summary>
        /// <param name="logEntry"></param>
        /// <returns>A boolean indicating if the item was logged for any contained logger.</returns>
        /// <remarks>
        /// Contract: pre  : logEntry must not be null.
        /// </remarks>
        protected override bool Log(LogEntry<T> logEntry)
        {
            bool result = false;

            if (CheckFilter(logEntry))
            {

                // Make a copy of the collection, so that access is to the collection is not blocked during the logging.
                // Logging items could be very expensive.
                ICollection<ILogger<T>> loggers = CloneLoggersCollection();

                foreach (ILogger<T> logger in loggers)
                {
                    result = (logger.Log(logEntry.Entry, logEntry.Severity, logEntry.Category) || result);
                }
            }

            return result;
        }

        /// <summary>
        /// Return a shallow cloned collection of logger objects.
        /// </summary>
        /// <returns></returns>
        /// <remarks>
        /// Contract: post : returned collection is not null.
        /// </remarks>
        protected virtual ICollection<ILogger<T>> CloneLoggersCollection()
        {
            Contract.Ensures(Contract.Result<ICollection<ILogger<T>>>() != null);

            ICollection<ILogger<T>> loggers;
            lock (LoggersLockObject)
            {
                loggers = new List<ILogger<T>>(Loggers);
            }

            return loggers;
        }

        #endregion

        #region Contracts

        [ContractInvariantMethod]
        private void ObjectInvariant()
        {
            Contract.Invariant(_Loggers != null);
            Contract.Invariant(_LoggersLockObject != null);
        }

        #endregion

    }


}
