﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics.Contracts;

namespace tpts.Logging
{
    /// <summary>
    /// Rollover a file name based on the current date.
    /// </summary>
    public class DateRolloverStrategy : IFileRolloverStrategy
    {

        #region Constants

        private const string DEFAULT_DATE_FORMAT = "yyyyMMdd";

        #endregion

        #region Declarations

        private readonly string _DateFormat;

        #endregion

        #region Constructors

        /// <summary>
        /// Default constructor for DateRolloverStrategy, date format defaults to yyyyMMdd
        /// </summary>
        public DateRolloverStrategy()
        {
            _DateFormat = DEFAULT_DATE_FORMAT; 
        }

        /// <summary>
        /// Constructor for DateRolloverStrategy, specifying the date format to use.
        /// </summary>
        public DateRolloverStrategy(string dateFormat)
        {
            Contract.Requires(dateFormat != null);

            _DateFormat = dateFormat;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Get the date format for the rolled over file name.
        /// </summary>
        /// <remarks>
        /// Contract: post : get DateFormat will not be null.
        /// </remarks>
        protected virtual string DateFormat
        {
            get
            {
                Contract.Ensures(Contract.Result<string>() != null);

                return _DateFormat;
            }
        }

        #endregion

        #region Public Methods
        #endregion

        #region Protected Methods
        #endregion

        #region Private Methods
        #endregion

        #region IFileRolloverStrategy

        /// <summary>
        /// Get the rolled file name based on the supplied file name.
        /// </summary>
        /// <param name="fileName"></param>
        /// <returns></returns>
        /// <remarks>
        /// Contract: pre  : fileName must not be null or empty or contain just whitespaces.
        ///           post : Result will not be null or empty or contain just whitespaces.
        /// </remarks>
        public string GetRolledFileName(string fileName)
        {
            string ext = System.IO.Path.GetExtension(fileName);
            int len = fileName.Length - ext.Length;
            string rolledFileName = "";
            if (0 < len) // len should always be greater than 1, here to satisfy contract for string.Substring.
            {
                rolledFileName= fileName.Substring(0, len) + DateTime.Today.ToString(DateFormat) + ext;
            }
            else
            {
                rolledFileName=DateTime.Today.ToString(DateFormat) + ext;
            }
            if (string.IsNullOrWhiteSpace(rolledFileName)) // This is to prove the post conditions.
            {
                return fileName;
            }
            else
            {
                return rolledFileName;
            }

        }

        #endregion

        #region Contracts

        [ContractInvariantMethod]
        private void ObjectInvariant()
        {
            Contract.Invariant(_DateFormat != null);
        }

        #endregion

    }


}
