﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.Diagnostics.Contracts;

namespace tpts.Logging
{
    /// <summary>
    /// Log items to the Windows Event Log.
    /// </summary>
    /// <typeparam name="T">The type of item to be logged.</typeparam>
    public class EventLogLogger<T> : Logger<T>
    {

        #region Constants

        private const string EVENT_LOG_NAME = "Application";  // windows event log name.
        private const int EVENT_LOG_EVENT_ID = 1; // ID number for the event log entry.

        #endregion

        #region Declarations

        private readonly string _ApplicationName;

        #endregion

        #region Constructors

        /// <summary>
        /// Constructor for EventLogLogger
        /// </summary>
        /// <param name="applicationName"></param>
        /// <remarks>
        /// Contract: pre  : application name must not be null, empty or contain just whitespaces.
        /// </remarks>
        public EventLogLogger(string applicationName)
        {
            Contract.Requires(!string.IsNullOrWhiteSpace(applicationName));
            Contract.Requires(applicationName != null);
            Contract.Requires(applicationName.Length > 0);

            _ApplicationName = applicationName;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Get the application name (source) for the Windows event log logger.
        /// </summary>
        /// <remarks>
        /// Contract: post : get ApplicationName is not null, empty or contain just whitespaces.
        /// </remarks>
        protected virtual string ApplicationName
        {
            [Pure()]
            get
            {
                Contract.Ensures(!string.IsNullOrWhiteSpace(Contract.Result<string>()));
                Contract.Ensures(Contract.Result<string>() != null);
                Contract.Ensures(Contract.Result<string>().Length > 0);

                return _ApplicationName;
            }
        }

        #endregion

        #region Public Methods
        #endregion

        #region Protected Methods

        /// <summary>
        /// Convert the log entry severity into a Windows event log entry type
        /// </summary>
        /// <param name="severity"></param>
        /// <returns></returns>
        protected virtual EventLogEntryType ConvertSeverityToEventType(LogEntrySeverity severity)
        {
            EventLogEntryType eventLogEntryType = EventLogEntryType.Information;
            switch (severity)
            {
                case LogEntrySeverity.Critical:
                case LogEntrySeverity.Error:
                case LogEntrySeverity.Fatal:
                    eventLogEntryType = EventLogEntryType.Error;
                    break;
                case LogEntrySeverity.Warning:
                    eventLogEntryType = EventLogEntryType.Warning;
                    break;
                default:
                    eventLogEntryType = EventLogEntryType.Information;
                    break;
            }

            return eventLogEntryType;
        }

        #endregion

        #region Private Methods
        #endregion

        #region Logger

        /// <summary>
        /// Log the entry
        /// </summary>
        /// <param name="logEntry"></param>
        /// <returns>A boolean indicating if the item was logged or not.</returns>
        /// <remarks>
        /// Contract: pre  : logEntry must not be null.
        /// </remarks>
        protected override bool Log(LogEntry<T> logEntry)
        {
            // Check log entry matches filter.
            if (CheckFilter(logEntry))
            {
                // Apply formatting
                string formattedEntry = FormatEntry(logEntry);

                // Write to Windows event log.
                EventLog.WriteEntry(_ApplicationName, formattedEntry, ConvertSeverityToEventType(logEntry.Severity), EVENT_LOG_EVENT_ID);

                return true;
            }
            else
            {
                return false;
            }
        }

        #endregion

        #region Contracts

        [ContractInvariantMethod]
        private void ObjectInvariant()
        {
            Contract.Invariant(!string.IsNullOrWhiteSpace(_ApplicationName));
            Contract.Invariant(_ApplicationName != null);
            Contract.Invariant(_ApplicationName.Length > 0);
        }

        #endregion
    }


}
