﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics.Contracts;

namespace tpts.Logging
{
    /// <summary>
    ///  Logger class to log items to a file.
    /// </summary>
    /// <typeparam name="T">The type of item to be logged.</typeparam>
    public class FileLogger<T> : Logger<T>
    {

        #region Declarations

        private readonly string _LogFileName;

        private readonly IFileRolloverStrategy _RolloverStrategy;

        private object _LogLockObject = new object(); // object used to synchronise access to file write calls.

        #endregion

        #region Constructors

        /// <summary>
        /// Constructor for File Logger
        /// </summary>
        /// <param name="logFileName"></param>
        /// <remarks>
        /// Contract: pre  : logFileName must not be null, empty or contain just white spaces.
        /// </remarks>
        public FileLogger(string logFileName)
        {
            Contract.Requires(!string.IsNullOrWhiteSpace(logFileName));
            Contract.Requires(logFileName != null);
            Contract.Requires(logFileName.Length > 0);

            _LogFileName = logFileName;
        }

        /// <summary>
        /// Constructor for File Logger, specifying the rollover strategy.
        /// </summary>
        /// <param name="logFileName"></param>
        /// <param name="rolloverStrategy"></param>
        /// <remarks>
        /// Contract: pre  : logFileName must not be null, empty or contain just white spaces.
        /// </remarks>
        public FileLogger(string logFileName, IFileRolloverStrategy rolloverStrategy)
        {
            Contract.Requires(!string.IsNullOrWhiteSpace(logFileName));
            Contract.Requires(logFileName != null);
            Contract.Requires(logFileName.Length > 0);

            _LogFileName = logFileName;
            _RolloverStrategy = rolloverStrategy;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Get the file name for the log file.
        /// </summary>
        /// <remarks>
        /// Contract: post : get LogFileName will not be null, empty or contain just whitespaces.
        /// </remarks>
        protected virtual string LogFileName
        {
            get
            {
                Contract.Ensures(!string.IsNullOrWhiteSpace(Contract.Result<string>()));
                Contract.Ensures(Contract.Result<string>() != null);
                Contract.Ensures(Contract.Result<string>().Length > 0);

                if (_RolloverStrategy != null) // If rollover strategy is specified then use it to get the log file name.
                {
                    return _RolloverStrategy.GetRolledFileName(_LogFileName);
                }
                else
                {
                    return _LogFileName;
                }
            }
        }

        /// <summary>
        /// Get the Rollover Strategy to use.
        /// </summary>
        protected virtual IFileRolloverStrategy RolloverStrategy
        {
            get
            {
                return _RolloverStrategy;
            }
        }

        /// <summary>
        /// Get the lock object used to synch access to LogFileName collection.
        /// </summary>
        /// <remarks>
        /// Contract: post : get LogFileNameLockObject will not be null.
        /// </remarks>
        protected virtual object LogLockObject
        {
            get
            {
                Contract.Ensures(Contract.Result<object>() != null);

                return _LogLockObject;
            }
        }

        #endregion

        #region Public Methods
        #endregion

        #region Protected Methods

        /// <summary>
        /// Write the text to the log file.
        /// </summary>
        /// <param name="entryText"></param>
        /// <remarks>
        /// Contract: pre  : entryText must not be null.
        /// </remarks>
        protected virtual void WriteEntryToFile(string entryText)
        {
            Contract.Requires(entryText != null);

            string fileName = "";
            lock (LogLockObject)
            {
                fileName = LogFileName;
                System.IO.File.AppendAllText(fileName, entryText);
            }
        }

        #endregion

        #region Private Methods
        #endregion

        /// <summary>
        /// Log the entry
        /// </summary>
        /// <param name="logEntry"></param>
        /// <returns>A boolean indicating if the item was logged or not.</returns>
        /// <remarks>
        /// Contract: pre  : logEntry must not be null.
        /// </remarks>
        protected override bool Log(LogEntry<T> logEntry)
        {
            // Check log entry matches filter.
            if (CheckFilter(logEntry))
            {
                // Apply formatting
                string formattedEntry = FormatEntry(logEntry);

                // Write to file.
                WriteEntryToFile(formattedEntry);

                return true;
            }
            else
            {
                return false;
            }
        }

        #region Contracts

        [ContractInvariantMethod]
        private void ObjectInvariant()
        {
            Contract.Invariant(!string.IsNullOrWhiteSpace(_LogFileName));
            Contract.Invariant(_LogFileName != null);
            Contract.Invariant(_LogFileName.Length > 0);
            Contract.Invariant(_LogLockObject != null);
        }

        #endregion
    }

}
