﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics.Contracts;

namespace tpts.Logging
{
    /// <summary>
    /// Rollover a file name based on file size.
    /// </summary>
    /// <remarks>
    /// Can override AppendFileCounter method to change the suffix used.
    /// </remarks>
    public class FileSizeRolloverStrategy : IFileRolloverStrategy
    {

        #region Declarations

        private readonly int _RolloverAtSize;  // File size to rollover when file exceeds (in MB).

        #endregion

        #region Constructors

        /// <summary>
        /// Constructor for FileSizeRolloverStrategy
        /// </summary>
        /// <param name="RolloverAtSize"></param>
        /// <remarks>
        /// Contract: pre  : maximumFileSize must be greater than 0.
        /// </remarks>
        public FileSizeRolloverStrategy(int RolloverAtSize)
        {
            Contract.Requires(RolloverAtSize >= 1);

            _RolloverAtSize = RolloverAtSize;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Get the file size at which to rollover the file (in MB).
        /// </summary>
        /// <remarks>
        /// Contract: post : get MaximumFileSize will be greater than 0 (zero).
        /// </remarks>
        protected virtual int RolloverAtSize
        {
            get
            {
                Contract.Ensures(Contract.Result<int>() >= 0);

                return _RolloverAtSize;
            }
        }

        #endregion

        #region Public Methods
        #endregion

        #region Protected Methods

        /// <summary>
        /// Append the counter value to the supplied file name.
        /// </summary>
        /// <param name="fileName"></param>
        /// <param name="counter"></param>
        /// <returns></returns>
        /// <remarks>
        /// Contract: pre  : fileName must not be null, empty or contain just whitespaces.
        ///           post : Result will not be null, empty or contain just whitespaces.
        /// </remarks>
        protected virtual string AppendFileCounter(string fileName, int counter)
        {
            Contract.Requires(!string.IsNullOrWhiteSpace(fileName));
            Contract.Requires(fileName != null);
            Contract.Requires(fileName.Length > 0);
            Contract.Ensures(!string.IsNullOrWhiteSpace(Contract.Result<string>()));
            Contract.Ensures(Contract.Result<string>() != null);
            Contract.Ensures(Contract.Result<string>().Length > 0);

            string ext = System.IO.Path.GetExtension(fileName);
            int len = fileName.Length - ext.Length;
            string rolledFileName = "";
            if (0 < len) // len should always be greater than 1, here to satisfy contract for string.Substring.
            {
                rolledFileName= fileName.Substring(0, len) + counter.ToString() + ext;
            }
            else
            {
                rolledFileName= counter.ToString() + ext;
            }
            if (string.IsNullOrWhiteSpace(rolledFileName)) // This is to prove the post conditions.
            {
                return fileName;
            }
            else
            {
                return rolledFileName;
            }
        }

        /// <summary>
        /// Get the size in bytes of the file.  If the file does not exist zero is returned.
        /// </summary>
        /// <param name="fileName"></param>
        /// <returns></returns>
        /// <remarks>
        /// Contract: pre  : fileName must not be null, empty or contain just whitespaces.
        ///           post : Result will be a long equal to or greater than 0 (zero).
        /// </remarks>
        protected long GetFileSize(string fileName)
        {
            Contract.Requires(!string.IsNullOrWhiteSpace(fileName));
            Contract.Requires(fileName != null);
            Contract.Requires(fileName.Length > 0);
            Contract.Ensures(Contract.Result<long>() >= 0);

            long fileSize = 0;
            try
            {
                if (System.IO.File.Exists(fileName))
                {
                    System.IO.FileInfo fileInfo = new System.IO.FileInfo(fileName);
                    fileSize = fileInfo.Length;
                }
            }
            catch
            {
                fileSize = 0;
            }

            return fileSize;

        }
        #endregion

        #region Private Methods
        #endregion

        #region IFileRolloverStrategy

        /// <summary>
        /// Get the rolled file name based on the supplied file name.
        /// </summary>
        /// <param name="fileName"></param>
        /// <returns></returns>
        /// <remarks>
        /// Contract: pre  : fileName must not be null or empty or contain just whitespaces.
        ///           post : Result will not be null or empty or contain just whitespaces.
        /// </remarks>
        public string GetRolledFileName(string fileName)
        {
            int maxFileSizeInBytes = RolloverAtSize * 1024;

            if (GetFileSize(fileName) >= maxFileSizeInBytes)
            {
                int index = 0;
                string rolledFileName = fileName;
                while (true)
                {
                    index++;
                    rolledFileName = AppendFileCounter(fileName, index);
                    if (GetFileSize(rolledFileName) <= maxFileSizeInBytes)
                    {
                        break;
                    }
                }

                return rolledFileName;
            }
            else
            {
                return fileName;
            }
        }

        #endregion

        #region Contracts

        [ContractInvariantMethod]
        private void ObjectInvariant()
        {
            Contract.Invariant(_RolloverAtSize > 0);
        }

        #endregion

    }

}
