﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics.Contracts;

namespace tpts.Logging
{
    /// <summary>
    /// Interface for a Logger for the specified object type.
    /// </summary>
    /// <typeparam name="T">The type of item to be logged.</typeparam>
    [ContractClass(typeof(ILoggerContract<>))]
    public interface ILogger<T>
    {

        /// <summary>
        /// Set the formatter to use for log entries.
        /// </summary>
        ILogEntryFormatter<T> LogEntryFormatter { set; }

        /// <summary>
        /// Set the filter to use for log entries.
        /// </summary>
        ILogEntryFilter<T> LogEntryFilter { set; }


        /// <summary>
        /// Log the entry.
        /// </summary>
        /// <param name="entry"></param>
        /// <returns>A boolean indicating if the item was logged or not.</returns>
        /// <remarks>
        /// Contract: pre  : entry must not be null.
        ///           post : entry will be logged with a entered date/time of the current date/time.
        ///                  entry will be logged with a severity of information.
        ///                  entry will be logged with a category of blank.
        /// </remarks>
        bool Log(T entry);

        /// <summary>
        /// Log the entry.
        /// </summary>
        /// <param name="entry"></param>
        /// <param name="category"></param>
        /// <returns>A boolean indicating if the item was logged or not.</returns>
        /// <remarks>
        /// Contract: pre  : entry must not be null.
        ///                  category must not be null.
        ///           post : entry will be logged with a entered date/time of the current date/time.
        ///                  entry will be logged with a severity of information.
        /// </remarks>
        bool Log(T entry, string category);

        /// <summary>
        /// Log the entry.
        /// </summary>
        /// <param name="entry"></param>
        /// <param name="severity"></param>
        /// <returns>A boolean indicating if the item was logged or not.</returns>
        /// <remarks>
        /// Contract: pre  : entry must not be null.
        ///           post : entry will be logged with a entered date/time of the current date/time.
        ///                  entry will be logged with a category of blank.
        /// </remarks>
        bool Log(T entry, LogEntrySeverity severity);

        /// <summary>
        /// Log the entry.
        /// </summary>
        /// <param name="entry"></param>
        /// <param name="severity"></param>
        /// <param name="category"></param>
        /// <returns>A boolean indicating if the item was logged or not.</returns>
        /// <remarks>
        /// Contract: pre  : entry must not be null.
        ///                  category must not be null.
        ///           post : entry will be logged with a entered date/time of the current date/time.
        /// </remarks>
        bool Log(T entry, LogEntrySeverity severity, string category);

    }
}
