﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics.Contracts;

namespace tpts.Logging
{
    /// <summary>
    /// Class representing a log entry of type T. Is immutable.
    /// </summary>
    /// <typeparam name="T">The type of item logged.</typeparam>
    /// <remarks>
    /// The getters for the log entry properties are public, the setters are protected to allow this class to be subclassed.
    /// 
    /// Contract: invar : Category is not null.
    ///                   Entry is not null.
    /// </remarks>
    public class LogEntry<T>
    {

        #region Declarations

        private readonly DateTime _LogEntryDateTime;
        private readonly T _Entry;
        private readonly string _Category = "";
        private readonly LogEntrySeverity _Severity;

        #endregion

        #region Constructors


        /// <summary>
        /// Constructor for Log Entry.
        /// </summary>
        /// <param name="entryDateTime">date/time stamp for entry being logged.</param>
        /// <param name="entry">entry object being logged.</param>
        /// <param name="severity">the severity of the entry.</param>
        /// <param name="category">the category for the entry.</param>
        /// <remarks>
        /// Contract: pre  : entry must not be null.
        ///                  category must not be null.
        /// </remarks>
        public LogEntry(DateTime entryDateTime, T entry, LogEntrySeverity severity, string category)
        {
            Contract.Requires(entry != null);
            Contract.Requires(category != null);

            _Entry = entry;
            _LogEntryDateTime = entryDateTime;
            _Category = category;
            _Severity = severity;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Get the date/time for the log entry.
        /// </summary>
        public virtual DateTime LogEntryDateTime
        {
            [Pure()]
            get { return _LogEntryDateTime; }
        }

        /// <summary>
        /// Get the category for the log entry.
        /// </summary>
        /// <remarks>
        ///
        /// Contract: post : get Category will not be null.
        /// </remarks>
        public virtual string Category
        {
            [Pure()]
            get
            {
                Contract.Ensures(Contract.Result<string>() != null);

                return _Category;
            }
        }

        /// <summary>
        /// Get the severity of the log entry.
        /// </summary>
        public virtual LogEntrySeverity Severity
        {
            [Pure()]
            get { return _Severity; }
        }

        /// <summary>
        /// Get the entry of the log entry.
        /// </summary>
        /// <remarks>
        /// Contract: post : get Entry will not be null.
        /// </remarks>
        public virtual T Entry
        {
            [Pure()]
            get
            {
                Contract.Ensures(Contract.Result<T>() != null);

                return _Entry;
            }
        }

        #endregion

        #region Public Methods
        #endregion

        #region Protected Methods
        #endregion

        #region Private Methods
        #endregion

        #region Contracts

        [ContractInvariantMethod]
        private void ObjectInvariant()
        {
            Contract.Invariant(this._Entry != null);
            Contract.Invariant(this._Category != null);
        }

        #endregion
    }
}
