﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics.Contracts;

namespace tpts.Logging
{
    /// <summary>
    /// The base class for loggers.
    /// </summary>
    /// <typeparam name="T">The type of item to be logged.</typeparam>
    [ContractClass(typeof(LoggerContract<>))]
    public abstract class Logger<T> : ILogger<T>
    {

        #region Constants

        private const LogEntrySeverity DEFAULT_SEVERITY = LogEntrySeverity.Information;
        private const string DEFAULT_CATEGORY = "";

        #endregion

        #region Declarations

        private ILogEntryFilter<T> _LogEntryFilter;
        private ILogEntryFormatter<T> _LogEntryFormatter;

        #endregion

        #region Constructors

        /// <summary>
        /// Default constructor for Logger.  Has no filter or formatter.
        /// </summary>
        public Logger()
        {
        }

        /// <summary>
        /// Constructor for Logger, specifying the filter and formatter.
        /// </summary>
        /// <param name="logEntryFilter"></param>
        /// <param name="logEntryFormatter"></param>
        public Logger(ILogEntryFilter<T> logEntryFilter, ILogEntryFormatter<T> logEntryFormatter)
        {
            LogEntryFilter = logEntryFilter;
            LogEntryFormatter = logEntryFormatter;
        }

        #endregion

        #region Properties

        #endregion

        #region Public Methods
        #endregion

        #region Protected Methods

        /// <summary>
        /// Return true if the log entry matches the filter criteria or if there is no filter specified for the logger.
        /// </summary>
        /// <param name="logEntry"></param>
        /// <returns></returns>
        /// <remarks>
        /// Contract: pre  : logEntry must not be null.
        /// </remarks>
        protected virtual bool CheckFilter(LogEntry<T> logEntry)
        {
            Contract.Requires(logEntry != null);

            if (LogEntryFilter != null)
            {
                return LogEntryFilter.Accept(logEntry);
            }
            else
            {
                return true;
            }
        }

        /// <summary>
        /// Format the log entry using logger formatter.
        /// </summary>
        /// <param name="logEntry"></param>
        /// <returns></returns>
        /// <remarks>
        /// Note: If no formatter specified then ToString is called on the log entry.
        /// 
        /// Contract: pre  : logEntry must not be null.
        /// </remarks>
        protected virtual string FormatEntry(LogEntry<T> logEntry)
        {
            Contract.Requires(logEntry != null);
            Contract.Ensures(Contract.Result<string>() != null);

            if (LogEntryFormatter != null)
            {
                return LogEntryFormatter.FormatEntry(logEntry);
            }
            else
            {
                return logEntry.ToString();
            }
        }

        #endregion

        #region Internal Methods

        /// <summary>
        /// Log the entry
        /// </summary>
        /// <param name="logEntry"></param>
        /// <returns>A boolean indicating if the item was logged or not.</returns>
        /// <remarks>
        /// Contract: pre  : logEntry must not be null.
        /// </remarks>
        protected abstract bool Log(LogEntry<T> logEntry);

        #endregion

        #region Private Methods
        #endregion

        #region ILogger

        /// <summary>
        /// Set the formatter to use for log entries.
        /// </summary>
        public virtual ILogEntryFormatter<T> LogEntryFormatter
        {
            protected get { return _LogEntryFormatter; }
            set { _LogEntryFormatter = value; }
        }

        /// <summary>
        /// Set the filter to use for log entries.
        /// </summary>
        public virtual ILogEntryFilter<T> LogEntryFilter
        {
            protected get { return _LogEntryFilter; }
            set { _LogEntryFilter = value; }
        }

        /// <summary>
        /// Log the entry.
        /// </summary>
        /// <param name="entry"></param>
        /// <returns>A boolean indicating if the item was logged or not.</returns>
        /// <remarks>
        /// Contract: pre  : entry must not be null.
        ///           post : entry will be logged with a entered date/time of the current date/time.
        ///                  entry will be logged with a severity of information.
        ///                  entry will be logged with a category of blank.
        /// </remarks>
        public virtual bool Log(T entry)
        {
            return Log(new LogEntry<T>(DateTime.Now, entry, DEFAULT_SEVERITY, DEFAULT_CATEGORY));
        }

        /// <summary>
        /// Log the entry.
        /// </summary>
        /// <param name="entry"></param>
        /// <param name="category"></param>
        /// <returns>A boolean indicating if the item was logged or not.</returns>
        /// <remarks>
        /// Contract: pre  : entry must not be null.
        ///                  category must not be null.
        ///           post : entry will be logged with a entered date/time of the current date/time.
        ///                  entry will be logged with a severity of information.
        /// </remarks>
        public virtual bool Log(T entry, string category)
        {
            return Log(new LogEntry<T>(DateTime.Now, entry, DEFAULT_SEVERITY, category));
        }

        /// <summary>
        /// Log the entry.
        /// </summary>
        /// <param name="entry"></param>
        /// <param name="severity"></param>
        /// <returns>A boolean indicating if the item was logged or not.</returns>
        /// <remarks>
        /// Contract: pre  : entry must not be null.
        ///           post : entry will be logged with a entered date/time of the current date/time.
        ///                  entry will be logged with a category of blank.
        /// </remarks>
        public virtual bool Log(T entry, LogEntrySeverity severity)
        {
            return Log(new LogEntry<T>(DateTime.Now, entry, severity, DEFAULT_CATEGORY));
        }

        /// <summary>
        /// Log the entry.
        /// </summary>
        /// <param name="entry"></param>
        /// <param name="severity"></param>
        /// <param name="category"></param>
        /// <returns>A boolean indicating if the item was logged or not.</returns>
        /// <remarks>
        /// Contract: pre  : entry must not be null.
        ///                  category must not be null.
        ///           post : entry will be logged with a entered date/time of the current date/time.
        /// </remarks>
        public virtual bool Log(T entry, LogEntrySeverity severity, string category)
        {
            return Log(new LogEntry<T>(DateTime.Now, entry, severity, category));
        }

        #endregion
    }


}
