﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.Specialized;
using System.Collections.ObjectModel;
using System.Diagnostics.Contracts;

namespace tpts.Logging
{

    using System.Collections;

    /// <summary>
    ///  Log items to a collection held in memory.
    /// </summary>
    /// <typeparam name="T">The type of item to be logged.</typeparam>
    public class MemoryLogger<T> : Logger<T>, IEnumerable, INotifyCollectionChanged
    {

        #region Events

        /// <summary>
        /// The collection changed event for the collection of Log Entries.
        /// </summary>
        public event NotifyCollectionChangedEventHandler CollectionChanged;

        #endregion

        #region Declarations

        private ICollection<ReadOnlyLogEntry> _LogEntries;

        private object _LogEntriesLockObject = new object(); // object used to synchronise access to _LogEntries collection.

        #endregion

        #region Constructors

        /// <summary>
        /// Constructor for Memory LogEntry.
        /// </summary>
        public MemoryLogger()
        {
            LogEntries = CreateLogEntriesCollection();
        }

        #endregion

        #region Properties

        /// <summary>
        /// Get/Set the LogEntries collection.
        /// </summary>
        /// <remarks>
        /// Contract: pre  : set value must not be null
        ///           post : get LogEntries will not be null.
        /// </remarks>
        protected ICollection<ReadOnlyLogEntry> LogEntries
        {
            get
            {
                Contract.Ensures(Contract.Result<ICollection<ReadOnlyLogEntry>>() != null);

                return _LogEntries;
            }
            set
            {
                Contract.Requires(value != null);

                _LogEntries = value;
            }
        }

        /// <summary>
        /// Get/Set the lock object used to synch access to LogEntries collection.
        /// </summary>
        /// <remarks>
        /// Contract: pre  : set value must not be null
        ///           post : get LogEntriesLockObject will not be null.
        /// </remarks>
        protected object LogEntriesLockObject
        {
            get
            {
                Contract.Ensures(Contract.Result<object>() != null);

                return _LogEntriesLockObject;
            }
            set
            {
                Contract.Requires(value != null);
                _LogEntriesLockObject = value;
            }
        }

        /// <summary>
        /// Return the number of LogEntries contained in the composite LogEntry.
        /// </summary>
        /// <remarks>
        /// Contract: post : count is greater or equal to zero.
        /// </remarks>
        public int Count
        {
            get
            {
                Contract.Ensures(Contract.Result<int>() >= 0);

                lock (LogEntriesLockObject)
                {
                    return LogEntries.Count;
                }
            }
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// Get a list of the log entries currently stored in memory.
        /// </summary>
        /// <returns></returns>
        /// <remarks>
        /// Contract: post : returned collection is not null.
        /// Contract:      : returned collection does not equal LogEntries.
        /// </remarks>
        public ICollection<ReadOnlyLogEntry> GetCurrentEntries()
        {
            return CloneLogEntriesCollection();
        }

        /// <summary>
        /// Get the log entry at the specified zero based index.
        /// </summary>
        /// <returns></returns>
        /// <remarks>
        /// Contract: pre  : index must be greater than or equal to zero and less than Count
        /// </remarks>
        public ReadOnlyLogEntry ItemAt(int index)
        {
            Contract.Requires(index >= 0 && index < Count);

            return LogEntries.ElementAt(index);
        }

        /// <summary>
        /// Clear the collection of LogEntries.
        /// </summary>
        /// <remarks>
        /// Contract: post :  LogEntries count will be zero.
        /// </remarks>
        public virtual void Clear()
        {
            ClearLogEntries();
        }

        /// <summary>
        /// Return true if the collection of LogEntries contains the specified LogEntry.
        /// </summary>
        /// <param name="logEntry"></param>
        /// <returns></returns>
        /// <remarks>
        /// Contract: pre  : LogEntry must not be null.
        /// </remarks>
        [Pure()]
        public virtual bool Contains(ReadOnlyLogEntry logEntry)
        {
            Contract.Requires(logEntry != null);

            lock (LogEntriesLockObject)
            {
                return LogEntries.Contains(logEntry);
            }
        }

        /// <summary>
        /// Remove the LogEntry from the collection.
        /// </summary>
        /// <param name="logEntry"></param>
        /// <remarks>
        /// Contract: pre  : LogEntry must not be null.
        ///                : LogEntry must exist in collection.
        ///           post : LogEntry will not be in collection.
        /// </remarks>
        public virtual void Remove(ReadOnlyLogEntry logEntry)
        {
            Contract.Requires(logEntry != null);
            Contract.Requires(Contains(logEntry));
            Contract.Ensures(!Contains(logEntry));

            RemoveLogEntry(logEntry);
        }

        #endregion

        #region Protected Methods

        /// <summary>
        /// Return an empty collection to contain LogEntry objects.
        /// </summary>
        /// <returns></returns>
        /// <remarks>
        /// Contract: post : returned collection is not null
        ///                  returned collection is empty, i.e. count = 0.
        /// </remarks>
        protected ICollection<ReadOnlyLogEntry> CreateLogEntriesCollection()
        {
            Contract.Ensures(Contract.Result<ICollection<ReadOnlyLogEntry>>() != null);

            return new List<ReadOnlyLogEntry>();
        }

        /// <summary>
        /// Return a shallow cloned collection of log entry objects.
        /// </summary>
        /// <returns></returns>
        /// <remarks>
        /// Contract: post : returned collection is not null.
        /// </remarks>
        protected ICollection<ReadOnlyLogEntry> CloneLogEntriesCollection()
        {
            Contract.Ensures(Contract.Result<ICollection<ReadOnlyLogEntry>>() != null);

            ICollection<ReadOnlyLogEntry> logEntries;
            lock (LogEntriesLockObject)
            {
                logEntries = new List<ReadOnlyLogEntry>(LogEntries);
            }

            return logEntries;
        }

        /// <summary>
        /// Clear the collection of LogEntries.
        /// </summary>
        /// <remarks>
        /// This routine raises the collection changed event.  If overridden will
        /// Contract: post :  LogEntries count will be zero.
        /// </remarks>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Contracts", "Ensures")]
        protected void ClearLogEntries()
        {
            Contract.Ensures(_LogEntries.Count == 0);

            lock (LogEntriesLockObject)
            {
                LogEntries.Clear();
            }
            OnLogEntriesCollectionChanged(new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Reset));
        }

        /// <summary>
        /// Add the log entry to the collection of log entries.
        /// </summary>
        /// <param name="logEntry"></param>
        /// <remarks>
        /// Contract: pre  :  logEntry must not be null.
        /// </remarks>
        protected void AddLogEntry(ReadOnlyLogEntry logEntry)
        {
            Contract.Requires(logEntry != null);

            int newIndex = -1;
            lock (LogEntriesLockObject)
            {
                LogEntries.Add(logEntry);
                newIndex = FindLogEntry(logEntry);
                if (newIndex!=-1)
                {
                    OnLogEntriesCollectionChanged(new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Add, logEntry, newIndex));
                }
            }
        }

        /// <summary>
        /// Returns true if the collection of LogEntries contains the specified LogEntry.
        /// </summary>
        /// <param name="LogEntry"></param>
        /// <returns></returns>
        /// <remarks>
        /// Contract: pre  : LogEntry must not be null.
        /// </remarks>
        [Pure()]
        protected bool ContainsLogEntry(ReadOnlyLogEntry LogEntry)
        {
            Contract.Requires(LogEntry != null);

            bool result = false;

            lock (LogEntriesLockObject)
            {
                result = LogEntries.Contains(LogEntry);
            }

            return result;
        }

        /// <summary>
        /// Remove the specified LogEntry to the collection.
        /// </summary>
        /// <param name="logEntry"></param>
        /// <remarks>
        /// Contract: pre  : LogEntry must not be null.
        ///           post : LogEntry will not be in collection.
        /// </remarks>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Contracts", "Ensures")]
        protected void RemoveLogEntry(ReadOnlyLogEntry logEntry)
        {
            Contract.Requires(logEntry != null);
            Contract.Ensures(!Contains(logEntry));

            int removedIndex = -1;
            lock (LogEntriesLockObject)
            {
                removedIndex = FindLogEntry(logEntry);
                if (removedIndex != -1)
                {
                    LogEntries.Remove(logEntry);
                }
            }
            if (removedIndex != -1)
            {
                OnLogEntriesCollectionChanged(new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Remove, logEntry, removedIndex));
            }
        }


        /// <remarks>
        /// Contract: pre  : logEntry must not be null.
        /// </remarks>
        protected int FindLogEntry(ReadOnlyLogEntry logEntry)
        {
            Contract.Requires(logEntry != null);

            int foundIndex = -1;
            lock (LogEntriesLockObject)
            {
                for (int index = 0; index < Count; index++)
                {
                    if (ItemAt(index).Equals(logEntry))
                    {
                        foundIndex = index;
                        break;
                    }
                }
            }

            return foundIndex;
        }

        /// <summary>
        /// Raise collection event changed.
        /// </summary>
        /// <param name="e"></param>
        /// <remarks>
        /// Contract: pre  : e must not be null.
        /// </remarks>
        protected void OnLogEntriesCollectionChanged(NotifyCollectionChangedEventArgs e)
        {
            Contract.Requires(e != null);

            NotifyCollectionChangedEventHandler collectionChanged = CollectionChanged;
            if (collectionChanged != null)
            {
                collectionChanged(this, e);
            }
        }

        #endregion

        #region LogEntry

        /// <summary>
        /// Log the entry to all the contained LogEntries.
        /// </summary>
        /// <param name="logEntry"></param>
        /// <returns>A boolean indicating if the item was logged for any contained LogEntry.</returns>
        /// <remarks>
        /// Contract: pre  : logEntry must not be null.
        /// </remarks>
        protected override bool Log(LogEntry<T> logEntry)
        {
            bool result = false;

            if (CheckFilter(logEntry))
            {
                string formattedEntry = FormatEntry(logEntry);

                AddLogEntry(new ReadOnlyLogEntry(logEntry.LogEntryDateTime, formattedEntry, logEntry.Severity, logEntry.Category));
                result = true;
            }

            return result;
        }

        #endregion

        #region Contracts

        [ContractInvariantMethod]
        private void ObjectInvariant()
        {
            Contract.Invariant(_LogEntries != null);
            Contract.Invariant(_LogEntriesLockObject != null);
        }

        #endregion

        #region IEnumerator

        /// <summary>
        /// Get the enumerator for the collection of Log Entry objects.
        /// </summary>
        /// <returns></returns>
        public IEnumerator GetEnumerator()
        {
            ICollection<ReadOnlyLogEntry> entries = CloneLogEntriesCollection();
            return entries.GetEnumerator();
        }

        #endregion

    }
}
