﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics.Contracts;

namespace tpts.Logging
{
    /// <summary>
    /// 
    /// </summary>
    public class RollingFileLogger<T> : FileLogger<T>
    {

        #region Declarations

        private readonly FileLogger<T> _FileLogger;
        private IFileRolloverStrategy _RolloverStrategy;

        #endregion

        #region Constructors

        /// <summary>
        /// Constructor for Rolling File Logger.
        /// </summary>
        /// <param name="fileLogger"></param>
        /// <param name="rolloverStrategy"></param>
        /// <remarks>
        /// Contract: pre  : fileLogger must not be null.
        ///                  rolloverStrategy must not be null.
        /// </remarks>
        public RollingFileLogger(FileLogger<T> fileLogger, IFileRolloverStrategy rolloverStrategy)
            : base(fileLogger.LogFileName)
        {
            Contract.Requires(fileLogger != null);
            Contract.Requires(rolloverStrategy != null);

            _FileLogger = fileLogger;
            RolloverStrategy = rolloverStrategy;
            _RolloverStrategy = rolloverStrategy;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Get the file logger to roll over.
        /// </summary>
        /// <remarks>
        /// Contract: post : get FileLogger will not be null.
        /// </remarks>
        protected virtual FileLogger<T> FileLogger
        {
            get
            {
                Contract.Ensures(Contract.Result<FileLogger<T>>() != null);

                return _FileLogger;
            }
        }

        /// <summary>
        /// Get/Set the rollover strategy to use.
        /// </summary>
        /// <remarks>
        /// Contract: pre  : set value must not be null.
        ///           post : get RolloverStrategy will not be null.
        /// </remarks>
        protected virtual IFileRolloverStrategy RolloverStrategy
        {
            get
            {
                Contract.Ensures(Contract.Result<IFileRolloverStrategy>() != null);

                return _RolloverStrategy;
            }
            set
            {
                Contract.Requires(value != null);

                _RolloverStrategy = value;
            }
        }

        /// <summary>
        /// Get the file name for the log file based on the wrapped FileLogger instance.
        /// </summary>
        /// <remarks>
        /// Contract: post : get LogFileName will not be null, empty or contain just whitespaces.
        /// </remarks>
        public override string LogFileName
        {
            get
            {
                return RolloverStrategy.GetRolledFileName(_FileLogger.LogFileName);
            }
        }

        #endregion

        #region Public Methods
        #endregion

        #region Protected Methods
        #endregion

        #region Private Methods
        #endregion

        #region Contracts

        [ContractInvariantMethod]
        private void ObjectInvariant()
        {
            Contract.Invariant(_FileLogger != null);
            Contract.Invariant(_RolloverStrategy != null);
        }

        #endregion

    }


}
