﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics.Contracts;

namespace tpts.Logging
{
    /// <summary>
    /// Class to format a log Entry in a standard way.
    /// </summary>
    /// <typeparam name="T">The type of item logged.</typeparam>
    public class StandardLogEntryFormatter<T> : ILogEntryFormatter<T>
    {

        #region Constants

        private const string DEFAULT_DATETIME_FORMAT = "G"; // General long date/time format from regional settings.

        #endregion

        #region Declarations

        private readonly string _DateTimeFormat;

        #endregion

        #region Constructors

        /// <summary>
        /// Default constructor for StandardLogEntryFormatter
        /// </summary>
        public StandardLogEntryFormatter()
        {
            _DateTimeFormat = DEFAULT_DATETIME_FORMAT;
        }

        /// <summary>
        /// Constructor for StandardLogEntryFormatter, specifying the date/time format.
        /// </summary>
        /// <remarks>
        /// Contract: pre  : dateTimeFormat must not be null or empty.
        /// </remarks>
        public StandardLogEntryFormatter(string dateTimeFormat)
        {
            Contract.Requires(!string.IsNullOrEmpty(dateTimeFormat));
            Contract.Requires(dateTimeFormat != null);
            Contract.Requires(dateTimeFormat.Length > 0);

            _DateTimeFormat = dateTimeFormat;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Get/Set the date format to be used when formatting log entries.
        /// </summary>
        /// <remarks>
        /// Contract: post  : DateTimeFormat must not be null or empty.
        /// </remarks>
        public virtual string DateTimeFormat
        {
            get
            {
                Contract.Ensures(!string.IsNullOrEmpty(Contract.Result<string>()));
                Contract.Ensures(Contract.Result<string>() != null);
                Contract.Ensures(Contract.Result<string>().Length > 0);

                return _DateTimeFormat;
            }
        }

        #endregion

        #region ILogEntryFormatter

        /// <summary>
        /// Format log entry with standard format.
        /// </summary>
        /// <param name="logEntry"></param>
        /// <returns></returns>
        public string FormatEntry(LogEntry<T> logEntry)
        {
            return logEntry.LogEntryDateTime.ToString(DateTimeFormat) + " " +
                logEntry.Severity.ToString() + " " +
                logEntry.Category + " " +
                logEntry.Entry.ToString();
        }

        #endregion

        #region Contracts

        [ContractInvariantMethod]
        private void ObjectInvariant()
        {
            Contract.Invariant(!string.IsNullOrEmpty(_DateTimeFormat));
            Contract.Invariant(_DateTimeFormat != null);
            Contract.Invariant(_DateTimeFormat.Length > 0);
        }

        #endregion

    }
}
