﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics.Contracts;
using System.Diagnostics.CodeAnalysis;

namespace tpts.Logging
{
    /// <summary>
    /// Logger for logging xml items in an efficient manner.
    /// </summary>
    /// <typeparam name="T">The type of item to be logged.</typeparam>
    public class XMLFileLogger<T> : FileLogger<T>
    {

        #region Constants

        private const string DEFAULT_HEADER_FILE_CONTENTS = "<?xml version=\"1.0\" standalone=\"yes\" ?>\r\n<!DOCTYPE errors [\r\n  <!ELEMENT entries (ANY)>\r\n  <!ENTITY entryList SYSTEM \"<datafilepath>\">\r\n]>\r\n<entries>\r\n&entryList;\r\n</entries>";
        private string DEFAULT_DATA_FILE_SUFFIX = "_Entries";


        #endregion

        #region Declarations

        private string _HeaderFileContents;
        private string _DataFileSuffix;

        #endregion

        #region Constructors

        /// <summary>
        /// Constructor for File Logger
        /// </summary>
        /// <param name="logFileName"></param>
        /// <remarks>
        /// Contract: pre  : logFileName must not be null, empty or contain just white spaces.
        /// </remarks>
        public XMLFileLogger(string logFileName)
            : base(logFileName)
        {
            Contract.Requires(!string.IsNullOrWhiteSpace(logFileName));
            Contract.Requires(logFileName != null);
            Contract.Requires(logFileName.Length > 0);

            _HeaderFileContents = DEFAULT_HEADER_FILE_CONTENTS;
            _DataFileSuffix = DEFAULT_DATA_FILE_SUFFIX;
        }

        /// <summary>
        /// Constructor for File Logger, specifying the rollover strategy.
        /// </summary>
        /// <param name="logFileName"></param>
        /// <param name="rolloverStrategy"></param>
        /// <remarks>
        /// Contract: pre  : logFileName must not be null, empty or contain just white spaces.
        /// </remarks>
        public XMLFileLogger(string logFileName, IFileRolloverStrategy rolloverStrategy)
            : base(logFileName, rolloverStrategy)
        {
            Contract.Requires(!string.IsNullOrWhiteSpace(logFileName));
            Contract.Requires(logFileName != null);
            Contract.Requires(logFileName.Length > 0);

            _HeaderFileContents = DEFAULT_HEADER_FILE_CONTENTS;
            _DataFileSuffix = DEFAULT_DATA_FILE_SUFFIX;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Get/Set the contents for the header file.
        /// </summary>
        /// <remarks>
        /// Contract: pre  : set value must not be null or empty.
        ///           post : get HeaderFileContents will not be null or empty.
        /// </remarks>
        protected virtual string HeaderFileContents
        {
            get
            {
                Contract.Ensures(!string.IsNullOrEmpty(Contract.Result<string>()));
                Contract.Ensures(Contract.Result<string>() != null);
                Contract.Ensures(Contract.Result<string>().Length > 0);

                return _HeaderFileContents;
            }
            set
            {
                Contract.Requires(!string.IsNullOrEmpty(value));
                Contract.Requires(value != null);
                Contract.Requires(value.Length > 0);

                _HeaderFileContents = value;
            }
        }
        /// <summary>
        /// Get/Set the data file suffix.
        /// </summary>
        /// <remarks>
        /// Contract: pre  : set value can not be null, empty or just whitespaces.
        ///           post : get DataFileSuffix will not be null, empty or just whitespaces. 
        /// </remarks>
        protected virtual string DataFileSuffix
        {
            get
            {
                Contract.Ensures(!string.IsNullOrWhiteSpace(Contract.Result<string>()));
                Contract.Ensures(Contract.Result<string>() != null);
                Contract.Ensures(Contract.Result<string>().Length > 0);

                return _DataFileSuffix;
            }
            set
            {
                Contract.Requires(!string.IsNullOrWhiteSpace(value));
                Contract.Requires(value != null);
                Contract.Requires(value.Length > 0);

                _DataFileSuffix = value;
            }
        }

        /// <summary>
        /// Get the file name for the data log file name.
        /// </summary>
        /// <returns></returns>
        /// <remarks>
        /// This is the file where the items are logged.
        /// 
        /// Contract: post : Result will not be null, empty or contain just whitespaces.
        /// </remarks>
        protected virtual string DataFileName()
        {
            Contract.Ensures(!string.IsNullOrWhiteSpace(Contract.Result<string>()));
            Contract.Ensures(Contract.Result<string>() != null);
            Contract.Ensures(Contract.Result<string>().Length > 0);

            string fileName = base.LogFileName;
            string ext = System.IO.Path.GetExtension(fileName);
            int len = fileName.Length - ext.Length;
            string dataFileName = "";
            if (0 < len) // len should always be greater than 1, here to satisfy contract for string.Substring.
            {
                dataFileName = fileName.Substring(0, len) + DataFileSuffix + ext;
            }
            else
            {
                dataFileName = DataFileSuffix + ext;
            }
            if (string.IsNullOrWhiteSpace(dataFileName)) // Here to satisfy post condition. Should never occur
            {
                return fileName + DEFAULT_DATA_FILE_SUFFIX;
            }
            else
            {
                return dataFileName;
            }
        }
        #endregion

        #region Public Methods
        #endregion

        #region Protected Methods

        /// <summary>
        /// Write the text to the log file.
        /// </summary>
        /// <param name="entryText"></param>
        /// <remarks>
        /// Contract: pre  : entryText must not be null.
        /// </remarks>
        protected override void WriteEntryToFile(string entryText)
        {
            string fileName = "";
            lock (LogLockObject)
            {
                WriteHeaderFile();
                fileName = DataFileName();
                System.IO.File.AppendAllText(fileName, entryText);
            }
        }

        /// <summary>
        /// Create the header file, if required.
        /// </summary>
        protected virtual void WriteHeaderFile()
        {
            if (!System.IO.File.Exists(LogFileName))
            {
                string contents = HeaderFileContents;
                string token = "<datafilepath>";
                contents = ReplaceToken(contents, token, LogFileName);
                System.IO.File.WriteAllText(LogFileName, contents);
            }
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Replace the token with another in the string.
        /// </summary>
        /// <param name="str"></param>
        /// <param name="token"></param>
        /// <param name="replaceWith"></param>
        /// <returns></returns>
        /// <remarks>
        /// Note: It is possible for this routine to go into an endless loop if
        ///       token is contained in replaceWith.
        /// 
        /// Contract: pre  : str must not be null.
        ///                  token must not be null or empty.
        ///                  replaceWith must not be null.
        ///           post : Result has all occurances of token in str replaced by replaceWith.
        ///                  Result is not null.
        /// </remarks>
        private string ReplaceToken(string str, string token, string replaceWith)
        {
            Contract.Requires(str != null);
            Contract.Requires(!string.IsNullOrEmpty(token));
            Contract.Requires(token != null);
            Contract.Requires(token.Length > 0);
            Contract.Requires(replaceWith != null);
            Contract.Ensures(Contract.Result<string>() != null);

            string result = str;

            int length = token.Length;
            while (str.IndexOf(token, StringComparison.CurrentCultureIgnoreCase) >= 0)
            {
                int index = str.IndexOf(token, StringComparison.CurrentCultureIgnoreCase);
                if (index + length <= str.Length && length > 0 && index >= 0)
                {
                    str = str.Replace(str.Substring(index, length), replaceWith);
                }
            }
            return str;
        }

        #endregion

        #region Contracts

        [ContractInvariantMethod]
        private void ObjectInvariant()
        {
            Contract.Invariant(!string.IsNullOrEmpty(_HeaderFileContents));
            Contract.Invariant(_HeaderFileContents != null);
            Contract.Invariant(_HeaderFileContents.Length > 0);
            Contract.Invariant(_HeaderFileContents != null);
            Contract.Invariant(!string.IsNullOrWhiteSpace(_DataFileSuffix));
            Contract.Invariant(_DataFileSuffix != null);
            Contract.Invariant(_DataFileSuffix.Length > 0);
        }

        #endregion

    }


}
