﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics.Contracts;
using System.Xml;

namespace tpts.Logging
{
    /// <summary>
    ///  Format log entries in a XML format.
    /// </summary>
    /// <typeparam name="T">The type of item logged.</typeparam>
    public class XMLLogEntryFormatter<T> : ILogEntryFormatter<T>
    {

        #region Constants

        private const string DEFAULT_DATETIME_FORMAT = "G"; // General long date/time format from regional settings.

        private const string DEFAULT_ROOT_ELEMENT_NAME = "LogItem";

        #endregion

        #region Declarations

        private readonly string _DateTimeFormat = DEFAULT_DATETIME_FORMAT;

        private readonly string _RootElementName = DEFAULT_ROOT_ELEMENT_NAME;

        #endregion

        #region Constructors

        /// <summary>
        /// Default constructor for XMLLogEntryFormatter
        /// </summary>
        public XMLLogEntryFormatter()
        {
            _DateTimeFormat = DEFAULT_DATETIME_FORMAT;
        }

        /// <summary>
        /// Constructor for XMLLogEntryFormatter, specifying the date/time format.
        /// </summary>
        /// <param name="rootElementName"></param>
        /// <remarks>
        /// Contract: pre  : rootElementName must not be null, empty or just whitespaces.
        /// </remarks>
        public XMLLogEntryFormatter(string rootElementName)
        {
            Contract.Requires(!string.IsNullOrEmpty(rootElementName));
            Contract.Requires(rootElementName != null);
            Contract.Requires(rootElementName.Length > 0);

            _RootElementName = rootElementName;
            _DateTimeFormat = DEFAULT_DATETIME_FORMAT;
        }

        /// <summary>
        /// Constructor for XMLLogEntryFormatter, specifying the date/time format.
        /// </summary>
        /// <param name="rootElementName"></param>
        /// <param name="dateTimeFormat"></param>
        /// <remarks>
        /// Contract: pre  : rootElementName must not be null, empty or just whitespaces.
        ///                  dateTimeFormat must not be null or empty.
        /// </remarks>
        public XMLLogEntryFormatter(string rootElementName, string dateTimeFormat)
        {
            Contract.Requires(!string.IsNullOrEmpty(rootElementName));
            Contract.Requires(rootElementName != null);
            Contract.Requires(rootElementName.Length > 0);
            Contract.Requires(!string.IsNullOrEmpty(dateTimeFormat));
            Contract.Requires(dateTimeFormat != null);
            Contract.Requires(dateTimeFormat.Length > 0);

            _RootElementName = rootElementName;
            _DateTimeFormat = dateTimeFormat;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Get the date format to be used when formatting log entries.
        /// </summary>
        /// <remarks>
        /// Contract: post  : get DateTimeFormat will not be null or empty.
        /// </remarks>
        public virtual string DateTimeFormat
        {
            get
            {
                Contract.Ensures(!string.IsNullOrEmpty(Contract.Result<string>()));
                Contract.Ensures(Contract.Result<string>() != null);
                Contract.Ensures(Contract.Result<string>().Length > 0);

                return _DateTimeFormat;
            }
        }

        /// <summary>
        /// Get the name of the root element for the logged item.
        /// </summary>
        /// <remarks>
        /// Contract: post  : get RootElementName will not be null or empty or just whitespaces.
        /// </remarks>
        public virtual string RootElementName
        {
            get
            {
                Contract.Ensures(!string.IsNullOrWhiteSpace(Contract.Result<string>()));
                Contract.Ensures(Contract.Result<string>() != null);
                Contract.Ensures(Contract.Result<string>().Length > 0);

                return _RootElementName;
            }
        }
        #endregion

        #region Public Methods
        #endregion

        #region Protected Methods

        /// <summary>
        /// Build a XML node for the log entry.
        /// </summary>
        /// <param name="logEntry"></param>
        /// <returns></returns>
        /// <remarks>
        /// Contract: pre  : logEntry must not be null.
        ///           post : Result is not null.
        /// </remarks>
        protected virtual XmlElement BuildEventLogXML(LogEntry<T> logEntry)
        {
            Contract.Requires(logEntry != null);
            Contract.Ensures(Contract.Result<XmlElement>() != null);

            XmlDocument xmlDoc = new XmlDocument();
            XmlElement xmlRoot = xmlDoc.CreateElement(RootElementName);
            xmlRoot.AppendChild(CreateLoggedAtElement(logEntry, xmlDoc));
            xmlRoot.AppendChild(CreateEntryElement(logEntry, xmlDoc));
            xmlRoot.AppendChild(CreateSeverityElement(logEntry, xmlDoc));
            xmlRoot.AppendChild(CreateCategoryElement(logEntry, xmlDoc));

            return xmlRoot;
        }

        /// <summary>
        /// Return the element for the Logged At value.
        /// </summary>
        /// <param name="logEntry"></param>
        /// <param name="xmlDoc"></param>
        /// <remarks>
        /// Contract: pre  : logEntry must not be null.
        ///                  xmlDoc must not be null.
        ///           post : Result is not null.
        /// </remarks>
        protected virtual XmlElement CreateLoggedAtElement(LogEntry<T> logEntry, XmlDocument xmlDoc)
        {
            Contract.Requires(xmlDoc != null);
            Contract.Requires(logEntry != null);
            Contract.Ensures(Contract.Result<XmlElement>() != null);

            return CreateElement("LoggedAt", logEntry.LogEntryDateTime.ToString(DateTimeFormat), xmlDoc);
        }

        /// <summary>
        /// Return the element for the Entry value.
        /// </summary>
        /// <param name="logEntry"></param>
        /// <param name="xmlDoc"></param>
        /// <remarks>
        /// Contract: pre  : logEntry must not be null.
        ///                  xmlDoc must not be null.
        ///           post : Result is not null.
        /// </remarks>
        protected virtual XmlElement CreateEntryElement(LogEntry<T> logEntry, XmlDocument xmlDoc)
        {
            Contract.Requires(xmlDoc != null);
            Contract.Requires(logEntry != null);
            Contract.Ensures(Contract.Result<XmlElement>() != null);

            return CreateElement("Entry", logEntry.Entry.ToString(), xmlDoc);
        }

        /// <summary>
        /// Return the element for the Severity value.
        /// </summary>
        /// <param name="logEntry"></param>
        /// <param name="xmlDoc"></param>
        /// <remarks>
        /// Contract: pre  : logEntry must not be null.
        ///                  xmlDoc must not be null.
        ///           post : Result is not null.
        /// </remarks>
        protected virtual XmlElement CreateSeverityElement(LogEntry<T> logEntry, XmlDocument xmlDoc)
        {
            Contract.Requires(xmlDoc != null);
            Contract.Requires(logEntry != null);
            Contract.Ensures(Contract.Result<XmlElement>() != null);

            return CreateElement("Severity", logEntry.Severity.ToString(), xmlDoc);
        }

        /// <summary>
        /// Return the element for the Category value.
        /// </summary>
        /// <param name="logEntry"></param>
        /// <param name="xmlDoc"></param>
        /// <remarks>
        /// Contract: pre  : logEntry must not be null.
        ///                  xmlDoc must not be null.
        ///           post : Result is not null.
        /// </remarks>
        protected virtual XmlElement CreateCategoryElement(LogEntry<T> logEntry, XmlDocument xmlDoc)
        {
            Contract.Requires(xmlDoc != null);
            Contract.Requires(logEntry != null);
            Contract.Ensures(Contract.Result<XmlElement>() != null);

            return CreateElement("Category", logEntry.Category, xmlDoc);
        }

        /// <summary>
        /// Create a XML element for the specified name and optionally value 
        /// </summary>
        /// <param name="name"></param>
        /// <param name="value"></param>
        /// <param name="xmlDoc"></param>
        /// <returns></returns>
        /// <remarks>
        /// Contract: pre  : name must not be null or empty.
        ///                  xmlDOc must not be null.
        ///           post : Result is not null.
        /// </remarks>
        protected virtual XmlElement CreateElement(string name, string value, XmlDocument xmlDoc)
        {
            Contract.Requires(xmlDoc != null);
            Contract.Requires(!string.IsNullOrEmpty(name));
            Contract.Requires(name != null);
            Contract.Requires(name.Length > 0);
            Contract.Ensures(Contract.Result<XmlElement>() != null);

            XmlElement xmlElement = xmlDoc.CreateElement(name);
            if (!string.IsNullOrEmpty(value))
            {
                xmlElement.AppendChild(xmlDoc.CreateTextNode(value));
            }

            return xmlElement;
        }

        /// <summary>
        /// Returned the XML element as a string.
        /// </summary>
        /// <param name="xmlEntry"></param>
        /// <returns></returns>
        protected virtual string FormatXML(XmlElement xmlEntry)
        {
            Contract.Ensures(Contract.Result<string>() != null);

            System.IO.StringWriter sw = new System.IO.StringWriter();
            XmlTextWriter xw = new XmlTextWriter(sw);
            xw.Formatting = Formatting.Indented;
            xw.Indentation = 3;
            xw.IndentChar = ' ';
            xmlEntry.WriteTo(xw);
            Console.Write(sw.ToString());
            return sw.ToString();
        }

        #endregion

        #region Private Methods
        #endregion

        #region ILogEntryFormatter

        /// <summary>
        /// Format log entry in XML format.
        /// </summary>
        /// <param name="logEntry"></param>
        /// <returns></returns>
        public string FormatEntry(LogEntry<T> logEntry)
        {
            XmlElement xmlEntry = BuildEventLogXML(logEntry);
            return FormatXML(xmlEntry);
        }

        #endregion

        #region Contracts

        [ContractInvariantMethod]
        private void ObjectInvariant()
        {
            Contract.Invariant(!string.IsNullOrEmpty(_DateTimeFormat));
            Contract.Invariant(_DateTimeFormat != null);
            Contract.Invariant(_DateTimeFormat.Length > 0);
            Contract.Invariant(!string.IsNullOrWhiteSpace(_RootElementName));
            Contract.Invariant(_RootElementName != null);
            Contract.Invariant(_RootElementName.Length > 0);
        }

        #endregion

    }


}
